@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 #
 use Carp ();
 
-use Net::SNMP::Mixin::Util qw/ push_error /;
+use Net::SNMP::Mixin::Util qw/push_error get_init_slot/;
 
 # this module export config
 #
@@ -47,26 +47,26 @@ Net::SNMP::Mixin::System - mixin class for the mib-2 system-group values
 
 =head1 VERSION
 
-Version 0.12
+Version 0.14
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.14';
 
 =head1 SYNOPSIS
 
 A Net::SNMP mixin class for mib-II system-group info. It's just in the distribution to act as a blueprint for mixin authors.
 
   use Net::SNMP;
-  use Net::SNMP::Mixin qw/mixer init_mixins/;
+  use Net::SNMP::Mixin;
 
   my $session = Net::SNMP->session( -hostname => 'foo.bar.com' );
 
   $session->mixer('Net::SNMP::Mixin::System');
   $session->init_mixins;
-  snmp_dispatcher() if $session->nonblocking;
-
-  die $session->error if $session->error;
+  snmp_dispatcher();
+  $session->init_ok();
+  die $session->errors if $session->errors;
 
   my $system_group = $session->get_system_group;
 
@@ -95,8 +95,10 @@ Returns the mib-II system-group as a hash reference:
 
 sub get_system_group {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # just a shallow copy for shallow values
   return { %{ $session->{$prefix}{sysGroup} } };
@@ -112,11 +114,23 @@ Fetch the SNMP mib-II system-group values from the host. Don't call this method
 
 =cut
 
-sub _init {
-  my ($session, $reload) = @_;
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 1;
 
-  die "$prefix already initalized and reload not forced.\n"
-  	if $session->{$prefix}{__initialized} && not $reload;
+sub _init {
+  my ( $session, $reload ) = @_;
+  my $agent = $session->hostname;
+
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+      && get_init_slot($session)->{$prefix} == 0
+      && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # initialize the object system-group infos
   my $success = _fetch_system_group($session);
@@ -182,15 +196,11 @@ sub _system_group_cb {
 
   unless (defined $vbl) {
     if (my $err_msg = $session->error) {
-
-      # Net::SNMP looses sometimes error messages in nonblocking
-      # mode, so we save them in an extra buffer
       push_error($session, "$prefix: $err_msg");
     };
     return;
   }
 
-
   $session->{$prefix}{sysGroup}{sysDescr}    = $vbl->{ SYS_DESCR() };
   $session->{$prefix}{sysGroup}{sysObjectID} = $vbl->{ SYS_OBJECT_ID() };
   $session->{$prefix}{sysGroup}{sysUpTime}   = $vbl->{ SYS_UP_TIME() };
@@ -199,7 +209,8 @@ sub _system_group_cb {
   $session->{$prefix}{sysGroup}{sysLocation} = $vbl->{ SYS_LOCATION() };
   $session->{$prefix}{sysGroup}{sysServices} = $vbl->{ SYS_SERVICES() };
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
 
   return 1;
 }
@@ -221,7 +232,7 @@ Fixes are dependant upon their severity and my availablity. Should a fix not be
 Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2015 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
