@@ -12,7 +12,8 @@ use Net::SNMP ();
 # this module export config
 #
 use Sub::Exporter -setup =>
-  { exports => [qw/idx2val hex2octet normalize_mac push_error/], };
+  { exports => [qw/idx2val hex2octet normalize_mac push_error get_init_slot/],
+  };
 
 =head1 NAME
 
@@ -20,11 +21,11 @@ Net::SNMP::Mixin::Util - helper class for Net::SNMP mixins
 
 =head1 VERSION
 
-Version 0.12
+Version 0.14
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.14';
 
 =head1 SYNOPSIS
 
@@ -198,14 +199,12 @@ This utility routine helps the mixin authors to push an error into the buffer wi
 
 Dies if session isn't a Net::SNMP object or error_msg is missing.
 
-=back
-
 =cut
 
 sub push_error {
   my ( $session, $error_msg ) = @_;
 
-  die "missing attribute 'session'," unless defined $session;
+  die "missing attribute 'session',"   unless defined $session;
   die "missing attribute 'error_msg'," unless defined $error_msg;
 
   die "'session' isn't a Net::SNMP object,"
@@ -213,9 +212,33 @@ sub push_error {
 
   # prepare the error buffer if not already done
   $session->{'Net::SNMP::Mixin'}{errors} ||= [];
+  my @errors = @{ $session->{'Net::SNMP::Mixin'}{errors} };
 
-  # store the error_msg at the buffer end
-  push @{ $session->{'Net::SNMP::Mixin'}{errors} }, $error_msg;
+  # store the error_msg at the buffer end if not already in the buffer
+  push @{ $session->{'Net::SNMP::Mixin'}{errors} }, $error_msg
+      unless grep m/\Q$error_msg\E$/, @errors;
+}
+
+=item B<< get_init_slot() >>
+
+Helper method, defines and returns the init hash slot for all mixin modules.
+
+=back
+
+=cut
+
+sub get_init_slot {
+  my ($session) = @_;
+
+  die "missing attribute 'session'," unless defined $session;
+
+  die "'session' isn't a Net::SNMP object,"
+    unless ref $session && $session->isa('Net::SNMP');
+
+  $session->{'Net::SNMP::Mixin'}{init_jobs_left} = {}
+    unless exists $session->{'Net::SNMP::Mixin'}{init_jobs_left};
+
+  return $session->{'Net::SNMP::Mixin'}{init_jobs_left};
 }
 
 unless ( caller() ) {
@@ -240,7 +263,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2015 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
